@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.27 2002/03/21 11:32:35 bazsi Exp $ */
+ * $Id: io.c,v 1.30 2002/10/07 08:09:15 bazsi Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -368,7 +368,16 @@ static int do_recv(struct abstract_read **r, UINT32 length, UINT8 *buffer, abstr
 	}
 
 	for (;;) {
-		int res = recvfrom(closure->fd, buffer, length, 0, (struct sockaddr *) addr, (socklen_t *) addrlen);
+		int res;
+		
+		res = recvfrom(closure->fd, buffer, length, 0, (struct sockaddr *) addr, (socklen_t *) addrlen);
+
+		if (*addrlen == 2) {
+			/* HACK: this is a workaround of a Linux 2.2 & 2.4 bug,
+			 * it doesn't return anything sensible in sockaddr buf 
+			 */
+			((struct sockaddr *) addr)->sa_family = AF_UNIX;
+		}
 
 		if (res == 0) {
 			/* on SOCK_DGRAM protocols 0 returned bytes doesn't mean anything */
@@ -834,9 +843,10 @@ static int inet_address2sockaddr_in(struct address_info *c, int sasize, struct s
 		memcpy(sa, &a->sa, sizeof(a->sa));
 		return 1;
 	}
-	if (a->ip && !tcp_addr(in,
-				a->ip->length, a->ip->data,
-				a->port))
+	if (tcp_addr(in,
+		     a ? a->ip->length : 7, 
+		     a ? (char *) a->ip->data : "0.0.0.0",
+		     a->port))
 		return 0;
 	else
 		if (!a->ip && !tcp_addr(in, 0, NULL, a->port))
@@ -993,6 +1003,8 @@ struct unix_address_info *make_unix_address_c(const char *path)
 struct address_info *sockaddr2address_info(size_t addr_len,
                                            struct sockaddr *addr)
 {
+	if (addr_len == 0)
+		return NULL;
 	switch(addr->sa_family)
 	{
 	case AF_INET:
